@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2019 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2020 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -186,6 +186,16 @@ extern char** environ;
 #define WRAPPER_ACTION_SOURCE_CODE_SIGNAL                  5  /* Action originated from a signal. */
 #define WRAPPER_ACTION_SOURCE_CODE_PING_TIMEOUT            11 /* Action originated from a timeout. */
 
+/* Required services access rights per action
+ * ATTENTION: Any change in the definitions below should be reported to the functions & variables in wrapper_service_permission_win.c  */
+#ifdef WIN32
+ #define WRAPPER_SERVICE_QUERY_STATUS        (SERVICE_QUERY_STATUS | SERVICE_QUERY_CONFIG)
+ #define WRAPPER_SERVICE_START               (SERVICE_QUERY_STATUS | SERVICE_START)
+ #define WRAPPER_SERVICE_STOP                (SERVICE_QUERY_STATUS | SERVICE_STOP)
+ #define WRAPPER_SERVICE_PAUSE_CONTINUE      (SERVICE_QUERY_STATUS | SERVICE_PAUSE_CONTINUE)
+ #define WRAPPER_SERVICE_CONTROL_CODE        (SERVICE_QUERY_STATUS | SERVICE_USER_DEFINED_CONTROL)
+#endif
+
 /* Because of the way time is converted to ticks, the largest possible timeout that
  *  can be specified without causing 32-bit overflows is (2^31 / 1000) - 5 = 2147478
  *  Which is a little over 24 days.  To make the interface nice, round this down to
@@ -206,6 +216,13 @@ typedef unsigned int TICKS;
 typedef unsigned long TICKS;
 #endif
 
+#define STATE_OUTPUT_MODE_DEFAULT   1
+#define STATE_OUTPUT_MODE_SECONDS   2
+#define STATE_OUTPUT_MODE_CHANGED   4
+
+#define STATE_OUTPUT_MASK_ALL       (STATE_OUTPUT_MODE_DEFAULT | STATE_OUTPUT_MODE_SECONDS | STATE_OUTPUT_MODE_CHANGED)
+#define STATE_OUTPUT_MASK_NOSECS    (STATE_OUTPUT_MASK_ALL & (~STATE_OUTPUT_MODE_SECONDS))
+
 #ifdef WIN32
 /* Defines the maximum number of service manager control events that can be queued in a single loop. */
 #define CTRL_CODE_QUEUE_SIZE 26 /* Can enqueue one less than this count at any time. */
@@ -221,7 +238,6 @@ typedef unsigned long TICKS;
 typedef struct PendingPing PendingPing, *PPendingPing;
 struct PendingPing {
     TICKS sentTicks;
-    TICKS timeoutTicks;
     TICKS slowTicks;
     PPendingPing nextPendingPing;
 };
@@ -229,10 +245,11 @@ struct PendingPing {
 /* Type definitions */
 typedef struct WrapperConfig WrapperConfig;
 struct WrapperConfig {
-    TCHAR   *argBinary;             /* The name of the wrapper binary. */
+    TCHAR   *argBinary;             /* The path to the wrapper binary. */
     TCHAR   *argCommand;            /* The command used to launch the wrapper. */
     TCHAR   *argCommandArg;         /* The argument to the command used to launch the wrapper. */
     TCHAR   *argConfFile;           /* The name of the config file from the command line. */
+    int     confFileOptional;       /* Whether the conf file is required or not. */
     TCHAR   *confDir;
     int     argConfFileDefault;     /* True if the config file was not specified. */
     int     argConfFileFound;       /* True if the config file was found. */
@@ -242,6 +259,7 @@ struct WrapperConfig {
     int     javaArgValueCount;      /* Number of the arguments getting passed over to the java application */
 
     TCHAR   *initialPath;           /* What the working directory was when the Wrapper process was first launched. */
+    TCHAR   *baseName;              /* The name of the wrapper binary without the OS, arch and bits. */
 #ifdef WIN32
     UINT    jvm_stdout_codepage;    /* The code page used for JVM outputs. */
 #endif
@@ -338,6 +356,7 @@ struct WrapperConfig {
     const TCHAR *nativeLibrary;     /* The base name of the native library loaded by the WrapperManager. */
     int     libraryPathAppendPath;  /* TRUE if the PATH environment variable should be appended to the java library path. */
     int     isStateOutputEnabled;   /* TRUE if set in the configuration file.  Shows output on the state of the state engine. */
+    int     stateOutputMode;        /* Mode used to print the state output. */
 #ifdef WIN32
     int     isMessageOutputEnabled; /* TRUE if detailed Message output should be included in debug output. */
 #endif
@@ -555,8 +574,8 @@ struct WrapperConfig {
 #define WRAPPER_PROCESS_DOWN      200
 #define WRAPPER_PROCESS_UP        201
 
-/* default timeout (in second) when executing child process to print java version */
-#define DEFAULT_JAVA_VERSION_TIMEOUT    10
+/* default timeout (in second) when executing child process to print java version (in 3.5.42 the value was increased from 10 to 30) */
+#define DEFAULT_JAVA_VERSION_TIMEOUT    30
 
 extern WrapperConfig *wrapperData;
 extern Properties    *properties;
@@ -756,11 +775,10 @@ extern int getOriginalWorkingDir();
 
 extern int wrapperLoadConfigurationProperties(int preload);
 
-extern void wrapperGetCurrentTime(struct timeb *timeBuffer);
-
 extern void updateStringValue(TCHAR **ptr, const TCHAR *value);
 
 #ifdef WIN32
+int confReadFilterCallbackMB(const char *bufferMB);
 
 extern void wrapperInitializeProfileCounters();
 extern void wrapperDumpPageFaultUsage();
@@ -782,15 +800,6 @@ extern void wrapperFreeStringArray(TCHAR **strings, int length);
 extern int wrapperInitialize(int silent);
 extern void wrapperDispose();
 
-/**
- * Returns the file name base as a newly malloced TCHAR *.  The resulting
- *  base file name will have any path and extension stripped.
- *
- * baseName should be long enough to always contain the base name.
- *  (strlen(fileName) + 1) is safe.
- */
-extern void wrapperGetFileBase(const TCHAR *fileName, TCHAR *baseName);
-
 /**
  * Output the version.
  */
@@ -822,6 +831,13 @@ extern void wrapperJVMProcessExited(TICKS nowTicks, int exitCode);
  */
 extern int wrapperConfirmJavaVersion();
 
+/**
+ * Set the Java version that the Wrapper will use.
+ *
+ * @param javaVersion The Java version to set or NULL if a default value should be set.
+ */
+void wrapperSetJavaVersion(JavaVersion* javaVersion);
+
 /**
  * Read and process any output from the child JVM Process.
  *
@@ -883,7 +899,10 @@ extern void wrapperCheckConsoleWindows();
  */
 extern void verifyEmbeddedSignature();
 
+extern TCHAR* getExceptionName(DWORD exCode, int nullOnUnknown);
+
 extern int exceptionFilterFunction(PEXCEPTION_POINTERS exceptionPointers);
+BOOL isSecondary();
 BOOL extern elevateThis(int argc, TCHAR **argv);
 BOOL extern duplicateSTD();
 BOOL extern myShellExec(HWND hwnd, LPCTSTR pszVerb, LPCTSTR pszPath, LPCTSTR pszParameters, LPCTSTR pszDirectory, TCHAR* namedPipeName);
@@ -893,7 +912,7 @@ BOOL extern isWin10OrHigher();
 BOOL extern isVista();
 BOOL extern isWinXP();
 extern void wrapperMaintainControlCodes();
-int wrapperServiceStatus(int consoleOutput);
+int wrapperServiceStatus(const TCHAR* serviceName, const TCHAR* serviceDisplayName, int consoleOutput);
 extern int wrapperRemove();
 #define OSBUFSIZE 256
 BOOL GetOSDisplayString(TCHAR** pszOS);
@@ -1118,6 +1137,8 @@ extern size_t wrapperQuoteValue(const TCHAR* value, TCHAR *buffer, size_t buffer
  */
 extern int wrapperCheckQuotes(const TCHAR *value, const TCHAR *propName);
 
+extern int getStateOutputModeForName(const TCHAR *name);
+
 /**
  * The main event loop for the wrapper.  Handles all state changes and events.
  */
